package com.platform.cac;

import com.platform.cac.tcp.CacClient;
import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.cac.tcp.message.dataframe.send.TcpRemoteAuthCacRequest;
import com.platform.info.GlobalData;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.util.JSONUtils;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author : shiyi
 * @Date : 2024/1/3 15:08
 * @Description : 对地面站业务逻辑的封装
 * <p>
 * // TODO 2024/11/28: 方法归类有点混乱，与地面站的交互和与中心指控的交互应该分开
 */
@Component
@Slf4j
public class GcsService {
    @Resource
    CacClient cacClient;
    @Resource
    RemoteService remoteService;


    public static Channel gcsChannel;

    // /**
    //  * 向地面站发送消息
    //  *
    //  * @param type 帧类型编码
    //  * @param data 数据内容（除去帧头、帧类型）
    //  * @return
    //  */
    // public ChannelFuture sendToGcs(byte type, byte[] data) {
    //     GcsMessage gcsMessage = new GcsMessage();
    //     gcsMessage.setHead(ClientTypeEnum.GCS.getWriteHead());
    //     gcsMessage.setType(type);
    //     gcsMessage.setReadableDataBytes(data);
    //     if (gcsChannel != null  && gcsChannel.isActive()) {
    //         return gcsChannel.writeAndFlush(gcsMessage);
    //     } else {
    //         log.warn("地面站消息发送失败: 与地面站握手未成功");
    //     }
    //     return null;
    // }
    // /**
    //  * 向地面站发送消息
    //  *
    //  * @param type 帧类型编码
    //  * @param buf  数据内容（除去帧头、帧类型）
    //  * @return
    //  */
    // public ChannelFuture sendToGcs(byte type, ByteBuf buf) {
    //     GcsMessage gcsMessage = new GcsMessage();
    //     gcsMessage.setHead(ClientTypeEnum.GCS.getWriteHead());
    //     gcsMessage.setType(type);
    //     gcsMessage.setReadableDataBytes(ByteBufUtil.getBytes(buf));
    //     buf.release();
    //     if (gcsChannel != null  && gcsChannel.isActive()) {
    //         return gcsChannel.writeAndFlush(gcsMessage);
    //     } else {
    //         log.warn("地面站消息发送失败: 与地面站握手未成功");
    //     }
    //     return null;
    // }

    /**
     * 构建指控系统的数据帧
     *
     * @param dataframe 基本数据帧
     * @param type      数据帧类型
     */
    public void buildRemoteBaseDataFrame(RemoteTcpBaseDataFrame dataframe, byte type) {
        dataframe.setMagicCode(GlobalData.REMOTE_HEAD);
        dataframe.setVersion(GlobalData.REMOTE_VERSION2);
        dataframe.setSerializationAlgorithm(GlobalData.REMOTE_SER_ALG);
        dataframe.setType(type);
        dataframe.setGcsIdLength((byte) GlobalData.GCS_ID.length());
        dataframe.setGcsId(GlobalData.GCS_ID);
        dataframe.setGcsAuthLength((byte) GlobalData.AUTHORIZATION.length());
        dataframe.setGcsAuth(GlobalData.AUTHORIZATION);
    }


    public boolean isSignIn() {
        if (GlobalData.GCS_SIGNIN) {
            return true;
        } else {
            log.warn("地面站未上线！正在重新上线...");
            gcsSignInRequest(); // 自动上线
            return false;
        }
    }


    public boolean gcsSignInRequest() {
        if (GlobalData.GCS_SIGNIN) {
            log.info("已登录中心指控，无需重复登录");
            return true;
        }
        Map<String, String> body = new HashMap<>();
        body.put("gcsId", GlobalData.GCS_ID);
        try {
            GlobalData.AUTHORIZATION = CacHpApi.gcsSignIn(JSONUtils.obj2json(body));
            if (GlobalData.AUTHORIZATION != null) {
                log.info("地面站上线请求成功，认证中...");
            }
            return GlobalData.AUTHORIZATION != null;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("地面站上线请求失败: {}", e.getMessage());
            return false;
        }

    }


    /**
     * 地面站上线tcp认证
     */
    public boolean gcsAuthRequestToCtrl() {
        TcpRemoteAuthCacRequest tcpGcsAuthCacRequest = new TcpRemoteAuthCacRequest();
        try {
            if (StringUtils.isEmpty(GlobalData.AUTHORIZATION)) {
                log.info("[tcpAuth] authorization 为空，重新发起上线请求");
                gcsSignInRequest();
            }
            buildRemoteBaseDataFrame(tcpGcsAuthCacRequest, RemoteFrameEnum.GCS_AUTH_CAC_REQUEST.getCode());
            tcpGcsAuthCacRequest.setUavIdLength((byte) 0);
            tcpGcsAuthCacRequest.setUavId("");
            tcpGcsAuthCacRequest.setReadableDataBytesLength((byte) 0);
            tcpGcsAuthCacRequest.setReadableDataBytes(null);

            if (CacClient.channelIsActive()) {
                ChannelFuture sendFuture = CacClient.write(tcpGcsAuthCacRequest);
                if (sendFuture != null && sendFuture.isSuccess()) {
                    log.info("[tcpAuth] 认证请求已成功发送，认证码：{}", GlobalData.AUTHORIZATION);
                    return true;
                } else {
                    log.error("[tcpAuth] 认证请求发送失败: {}", sendFuture != null ? sendFuture.cause() : "");
                }
            } else {
                log.error("[tcpAuth] 和中心指控之间tcp连接异常，tcp认证发送失败！");
                return false;
            }
        } catch (Exception e) {
            log.info("[tcpAuth] tcp认证数据构建失败 ｛｝", e);
        }
        // if (sendFuture == null || !sendFuture.isSuccess()) {
        //     // 如果发送失败，则重复验证
        //     cacClient.channel.eventLoop().schedule(this::gcsAuthRequestToCtrl, 3, TimeUnit.SECONDS);
        // }
        return false;
    }


}
