package com.platform.cac;


import com.platform.cac.tcp.CacClient;
import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.info.GlobalData;
import com.platform.info.enums.UavTypeEnum;
import com.platform.info.mapping.HaborUavMap;
import com.platform.info.mapping.UavIdMap;
import com.platform.info.dto.DirectControlUavParam;
import com.platform.util.JSONUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;


/**
 * @Author : shiyi
 * @Date : 2024/1/31 18:54
 * @Description : 中心指控提供的一些信息查询接口
 */
@Component
@Slf4j
public class RemoteService {
    @Resource
    CacClient cacClient;

    /**
     * 接收到运管通知的消息回报
     */
    public Boolean remoteResponse(RemoteTcpBaseDataFrame data, byte type) {
        try {
            ByteBuf bufToRemote = Unpooled.buffer();
            bufToRemote.writeShort(data.getMagicCode());
            bufToRemote.writeByte(data.getVersion());
            bufToRemote.writeByte(data.getSerializationAlgorithm());
            bufToRemote.writeByte(type);
            bufToRemote.writeByte(data.getGcsIdLength());
            bufToRemote.writeBytes(data.getGcsId().getBytes(StandardCharsets.UTF_8));
            bufToRemote.writeByte(data.getGcsAuthLength());
            bufToRemote.writeBytes(data.getGcsAuth().getBytes(StandardCharsets.UTF_8));
            bufToRemote.writeByte(data.getUavIdLength());
            if (data.getUavId() != null) {
                bufToRemote.writeBytes(data.getUavId().getBytes(StandardCharsets.UTF_8));
            }
            bufToRemote.writeByte(0x01);
            bufToRemote.writeInt(0);
            bufToRemote.writeByte(0x00);
            // cacClient.channel.writeAndFlush(bufToRemote);
            CacClient.write(bufToRemote);
            return true;
        } catch (Exception e) {
            log.error("运管报文回复失败", e);
            return false;
        }
    }

    /**
     * 根据哈勃序列号，查询映射关系，并更新缓存映射关系
     *
     * @return haborSn
     */
    public DirectControlUavParam queryCacDirectControlUavInfo(String haborSn) {
        DirectControlUavParam info = CacHpApi.queryCacDirectControlUav(haborSn);
        if (info == null) {
            return null;
        }
        UavIdMap.addMap(UavTypeEnum.getByRemoteCode(info.getUavType()), Integer.parseInt(info.getFlightControlSn()), info.getUavId());
        return info;
    }

    /**
     * 查询并更新当前地面站在控飞机的无人机映射关系, 需要不定期更新
     */
    public void queryCacDirectControlMapping() {
        Map<String, String> body = new HashMap<>();
        body.put("gcsId", GlobalData.GCS_ID);
        log.debug("查询全量无人机映射关系，request body: {}", body);
        try {
            List<DirectControlUavParam> directControlUavParamList = CacHpApi.queryAllCacDirectControlUavList(JSONUtils.obj2json(body));
            if (!CollectionUtils.isEmpty(directControlUavParamList)) {
                UavIdMap.clear();
                HaborUavMap.clear();
                directControlUavParamList.forEach(mapping -> {
                    UavTypeEnum uavType = UavTypeEnum.getByRemoteCode(mapping.getUavType());
                    // DONE 2024/6/26: 记录所有型号的飞机映射
                    UavIdMap.addMap(uavType, Integer.parseInt(mapping.getFlightControlSn()), mapping.getUavId());
                    HaborUavMap.addMap(mapping.getUavId(), mapping.getHaborSn());
                });
                log.info("直控无人机参数映射关系: {}", directControlUavParamList);
            }
            // if (UAV_TYPE != null) {
            //     log.info("当前{}型号无人机映射关系（fkId→uavId）：{}", UAV_TYPE, UavIdMap.showMap());
            // } else {
            //     log.info("无法获取当前无人机型号，所有型号无人机映射关系（fkId→uavId）为：{}", UavIdMap.showAllMap());
            // }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询全量无人机映射关系失败");
        }
    }

    /**
     * 查询地面站在控无人机
     */
    // public List<Integer> queryGcsControlUav() {
    //     List<String> uavIdList = queryGcsControlUavByUavId();
    //     if (uavIdList == null) {
    //         return null;
    //     }
    //     UavIdControlMap.clear();
    //     ArrayList<Integer> fkUavIdList = new ArrayList<>();
    //     for (String uavId : uavIdList) {
    //         // 如果uavId不属于当前地面站可控型号，则跳过
    //         if (UavIdMap.uavIdControllable(uavId)) {
    //             int fkId = UavIdMap.getFkId(uavId);
    //             fkUavIdList.add(fkId);
    //             UavIdControlMap.addMap(fkId, uavId);
    //         }
    //     }
    //     return fkUavIdList;
    // }

}
