package com.platform.cac.tcp;



import com.platform.cac.GcsService;
import com.platform.cac.tcp.codec.RemoteTcpBaseDataDecoder;
import com.platform.cac.tcp.codec.RemoteTcpBaseDataEncoder;
import com.platform.cac.tcp.message.RemoteMessageDispatcher;
import com.platform.config.CacRemoteConfig;
import com.platform.info.GlobalData;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.DelimiterBasedFrameDecoder;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.util.concurrent.TimeUnit;

/**
 * @Author : shiyi
 * @Date : 2023/12/30 16:57
 * @Description : 作为客户端，连接运管和中心指控
 */
@Slf4j
@Component
public class CacClient {
    @Resource
    private CacRemoteConfig cacRemoteConfig;

    @Resource
    CacConnectionHandler cacConnectionHandler;
    @Resource
    RemoteMessageDispatcher remoteMessageDispatcher;
    static EventLoopGroup eventLoopGroup = new NioEventLoopGroup();
    private Bootstrap bootstrap = new Bootstrap();

    private static Channel channel;

    @Resource
    GcsService gcsService;

    public void initClient(EventLoopGroup group) {

        ByteBuf delimiter = Unpooled.buffer();
        delimiter.writeShort(GlobalData.REMOTE_HEAD);
        if (null == group) {
            group = eventLoopGroup;
        }
        bootstrap.group(group)
                .channel(NioSocketChannel.class)
                .remoteAddress(new InetSocketAddress(cacRemoteConfig.getIp(), cacRemoteConfig.getRemoteTcpPort()))
                .handler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    public void initChannel(SocketChannel ch) throws Exception {
                        ch.pipeline().addLast("ping", new IdleStateHandler(10, 10, 10, TimeUnit.SECONDS));
                        ch.pipeline().addLast(new DelimiterBasedFrameDecoder(1024, true, true, delimiter));
                        ch.pipeline().addLast(new RemoteTcpBaseDataDecoder());
                        ch.pipeline().addLast(new RemoteTcpBaseDataEncoder());
                        ch.pipeline().addLast(cacConnectionHandler);
                        ch.pipeline().addLast(remoteMessageDispatcher);
                    }
                });
    }

    @PostConstruct
    public void start() {
        initClient(null);
        // ResourceLeakDetector.setLevel(ResourceLeakDetector.Level.ADVANCED);
        ChannelFuture future = bootstrap.connect().addListener((ChannelFuture futureListener) -> {
            if (!futureListener.isSuccess()) {
                log.info("与" + cacRemoteConfig.getIp() + ":" + cacRemoteConfig.getRemoteTcpPort() + "连接失败! 重连中...");
                connect();
            } else {
                channel = futureListener.channel();
                log.debug("start-channel: {}", channel);
            }
        });
    }
    /**
     * 连接指定的服务端，连接失败自动重连
     */
    public void connect() {
        ChannelFuture channelFuture = bootstrap.connect();
        // 使用最新的ChannelFuture -> 开启最新的监听器
        channelFuture.addListener((ChannelFutureListener) future -> {
            if (future.cause() != null) {
                log.info("与{}:{}连接失败! 重连中...", cacRemoteConfig.getIp(),cacRemoteConfig.getRemoteTcpPort() );
                future.channel().eventLoop().schedule(this::connect, 3, TimeUnit.SECONDS);
            } else {
                channel = future.channel();
                log.debug("reconnect-channel: {}", channel);
                if (GlobalData.GCS_SIGNIN) {
                    log.info("[reconnect] 与中心指控{}:{}重连成功, 认证中", cacRemoteConfig.getIp(),cacRemoteConfig.getRemoteTcpPort() );
                    gcsService.gcsAuthRequestToCtrl();
                }
            }
        });
    }

    public static ChannelFuture write(Object msg) {
        if (channelIsActive()){
            return CacClient.channel.writeAndFlush(msg);
        }
        return null;
    }

    public static boolean channelIsActive() {
        return CacClient.channel.isActive();

    }
    @PreDestroy
    private void end() {
        eventLoopGroup.shutdownGracefully();
    }
}