package com.platform.cac.tcp;


import com.platform.cac.GcsService;
import com.platform.cac.tcp.message.dataframe.send.TcpHeartBeatRequest;
import com.platform.info.GlobalData;
import com.platform.info.enums.GcsTypeEnum;
import com.platform.info.enums.RemoteFrameEnum;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.concurrent.TimeUnit;



/**
 * @Author : shiyi
 * @Date : 2024/1/3 14:14
 * @Description : 处理连接、断联、重连等逻辑
 */
@Slf4j
@Component
@ChannelHandler.Sharable
public class CacConnectionHandler extends ChannelInboundHandlerAdapter {
    @Resource
    CacClient cacClient;
    @Resource
    GcsService gcsService;


    /**
     * 建立连接时
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        InetSocketAddress ipSocket = (InetSocketAddress) ctx.channel().remoteAddress();
        int port = ipSocket.getPort();
        String host = ipSocket.getHostString();
        log.info("与中心指控{}:{}建立tcp连接!", host, port);
        ctx.channel().eventLoop().schedule(() -> {
                    log.info("地面站发送上线请求");
                    if (gcsService.gcsSignInRequest()) {
                        gcsService.gcsAuthRequestToCtrl();
                        sendHeartBeat();
                    }
                },
                1, TimeUnit.SECONDS);
        log.debug("connect-channel: {}", ctx.channel());
        ctx.fireChannelActive();
    }

    /**
     * 关闭连接时
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        InetSocketAddress ipSocket = (InetSocketAddress) ctx.channel().remoteAddress();
        int port = ipSocket.getPort();
        String host = ipSocket.getHostString();
        log.error("与中心指控{}:{}连接断开!", host, port);
        log.info("断线重连......");
        cacClient.connect();
    }


    /**
     * 心跳机制
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent) {
            IdleState state = ((IdleStateEvent) evt).state();
            if (IdleState.WRITER_IDLE.equals(state)) {
                //心跳
                TcpHeartBeatRequest tcpHeartBeatRequest = buildHeatBeatRequest();
                sendHeartBeat();
                sendHeartBeat(); // 加一条心跳作为分隔符
            }

        }
    }

    private void sendHeartBeat() {
        CacClient.write(buildHeatBeatRequest());
    }
    private TcpHeartBeatRequest buildHeatBeatRequest(){
        TcpHeartBeatRequest tcpHeartBeatRequest = new TcpHeartBeatRequest();
        tcpHeartBeatRequest.setMagicCode(GlobalData.REMOTE_HEAD);
        tcpHeartBeatRequest.setVersion(GlobalData.REMOTE_VERSION2);
        tcpHeartBeatRequest.setSerializationAlgorithm(GlobalData.REMOTE_SER_ALG);
        tcpHeartBeatRequest.setType(RemoteFrameEnum.HEART_BEAT_REQUEST.getCode());
        tcpHeartBeatRequest.setGcsIdLength((byte) GlobalData.GCS_ID.getBytes().length);
        tcpHeartBeatRequest.setGcsId(GlobalData.GCS_ID);
        tcpHeartBeatRequest.setGcsAuthLength((byte) GlobalData.GCS_TOKEN.getBytes().length);
        tcpHeartBeatRequest.setGcsAuth(GlobalData.GCS_TOKEN);
        tcpHeartBeatRequest.setUavIdLength((byte) 0);
        tcpHeartBeatRequest.setCurrentTime(System.currentTimeMillis());
        byte[] readableDataBytes = ByteBuffer.allocate(Long.SIZE / Byte.SIZE).putLong(tcpHeartBeatRequest.getCurrentTime()).array();
        tcpHeartBeatRequest.setReadableDataBytesLength(Long.SIZE / Byte.SIZE);
        tcpHeartBeatRequest.setReadableDataBytes(readableDataBytes);
        return tcpHeartBeatRequest;
    }
}
