package com.platform.cac.tcp.codec;



import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.info.GlobalData;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import lombok.extern.slf4j.Slf4j;

import java.util.List;


@Slf4j
public class RemoteTcpBaseDataDecoder extends ByteToMessageDecoder {
    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) {


        // byte[] buffer = new byte[in.readableBytes()];
        // in.readBytes(buffer);
        // in.release();
        // log.info("msg: {}", ByteUtils.bytes2HexString(buffer));

        RemoteTcpBaseDataFrame remoteTcpBaseDataFrame = new RemoteTcpBaseDataFrame();
        // 标记当前读取位置
        in.markReaderIndex();
        // 判断是否大于最小长度
        if (in.readableBytes() <= 8) {
            in.resetReaderIndex();
            log.error("[GcsTcpBaseDataDecoder][连接({}) 解析消息失败，数据可读长度小于数据帧最小长度in={}]", ctx.channel().id(), in.toString());
            in.clear();
            return;
        }

        byte[] totalIn = new byte[in.readableBytes()];
        in.readBytes(totalIn);
        // if (totalIn[2] != 0x04) {
        //     log.debug("完整消息: 88 99 {}", ByteUtils.bytes2HexString(totalIn));
        // }
        in.resetReaderIndex();

        remoteTcpBaseDataFrame.setMagicCode(GlobalData.REMOTE_HEAD);
        remoteTcpBaseDataFrame.setVersion(in.readByte());
        remoteTcpBaseDataFrame.setSerializationAlgorithm(in.readByte());
        remoteTcpBaseDataFrame.setType(in.readByte());
        // 获取地面站编号长度以及地面站编号
        byte gcsIdLengthByte = in.readByte();
        int gcsIdLength = Byte.toUnsignedInt(gcsIdLengthByte);
        if (in.readableBytes() < gcsIdLength) {
            in.resetReaderIndex();
            log.error("[GcsTcpBaseDataDecoder][连接({}) 解析消息失败，数据剩余可读长度小于gcsId长度，in={}]", ctx.channel().id(), in.toString());
            return;
        }
        byte[] gcsIdByteArray = new byte[gcsIdLength];
        in.readBytes(gcsIdByteArray);
        remoteTcpBaseDataFrame.setGcsIdLength(gcsIdLengthByte);
        remoteTcpBaseDataFrame.setGcsId(new String(gcsIdByteArray));
        // 获取地面站Token长度以及地面站Token
        byte gcsTokenLengthByte = in.readByte();
        int gcsTokenLength = Byte.toUnsignedInt(gcsTokenLengthByte);
        if (in.readableBytes() < gcsTokenLength) {
            in.resetReaderIndex();
            log.error("[GcsTcpBaseDataDecoder][连接({}) 解析消息失败，数据剩余可读长度小于gcsToken长度，in={}]", ctx.channel().id(), in.toString());
            return;
        }
        byte[] gcsTokenByteArray = new byte[gcsTokenLength];
        in.readBytes(gcsTokenByteArray);
        remoteTcpBaseDataFrame.setGcsAuthLength(gcsTokenLengthByte);
        remoteTcpBaseDataFrame.setGcsAuth(new String(gcsTokenByteArray));

        // 获取无人机编号长度以及无人机编号
        byte uavIdLengthByte = in.readByte();
        int uavIdLength = Byte.toUnsignedInt(uavIdLengthByte);
        if (in.readableBytes() < uavIdLength) {
            in.resetReaderIndex();
            log.error("[GcsTcpBaseDataDecoder][连接({}) 解析消息失败，数据剩余可读长度小于uavId长度，in={}]", ctx.channel().id(), in.toString());
            return;
        }
        if (uavIdLength == 0) {
            remoteTcpBaseDataFrame.setUavIdLength(uavIdLengthByte);
            remoteTcpBaseDataFrame.setUavId(null);
        } else {
            byte[] uavIdByteArray = new byte[uavIdLength];
            in.readBytes(uavIdByteArray);
            remoteTcpBaseDataFrame.setUavIdLength(uavIdLengthByte);
            remoteTcpBaseDataFrame.setUavId(new String(uavIdByteArray));
        }
        // 读取剩余字节
        remoteTcpBaseDataFrame.setReadableDataBytesLength(in.readableBytes());
        if (in.readableBytes() > 0) {
            byte[] readableDataBytes = new byte[in.readableBytes()];
            in.readBytes(readableDataBytes);
            remoteTcpBaseDataFrame.setReadableDataBytes(readableDataBytes);
        }
        out.add(remoteTcpBaseDataFrame);
        // TODO: 2023/6/14 测试之后记得删除此log
        //  log.info("[GcsTcpBaseDataDecoder][连接({}) 解析到一条消息({})]", ctx.channel().id(), gcsTcpBaseDataFrame.toString());
    }
}
