package com.platform.cac.tcp.codec;



import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import lombok.extern.slf4j.Slf4j;


@Slf4j
public class RemoteTcpBaseDataEncoder extends MessageToByteEncoder<RemoteTcpBaseDataFrame> {

    @Override
    protected void encode(ChannelHandlerContext ctx, RemoteTcpBaseDataFrame dataFrame, ByteBuf buf) {
        if (dataFrame != null) {
            buf.writeShort(dataFrame.getMagicCode());
            buf.writeByte(dataFrame.getVersion());
            buf.writeByte(dataFrame.getSerializationAlgorithm());
            buf.writeByte(dataFrame.getType());
            buf.writeByte(dataFrame.getGcsIdLength());
            if (dataFrame.getGcsIdLength() > 0) {
                buf.writeBytes(dataFrame.getGcsId().getBytes(), 0, dataFrame.getGcsIdLength());
            }
            buf.writeByte(dataFrame.getGcsAuthLength());
            if (dataFrame.getGcsAuthLength() > 0) {
                buf.writeBytes(dataFrame.getGcsAuth().getBytes(), 0, dataFrame.getGcsAuthLength());
            }
            buf.writeByte(dataFrame.getUavIdLength());
            if (dataFrame.getUavIdLength() > 0) {
                buf.writeBytes(dataFrame.getUavId().getBytes(), 0, dataFrame.getUavIdLength());
            }
            if (dataFrame.getReadableDataBytesLength() > 0) {
                buf.writeBytes(dataFrame.getReadableDataBytes(), 0, dataFrame.getReadableDataBytesLength());
            }
            // TODO: 2023/6/13 测试之后记得删除此log
            // log.debug("[encode]连接({}) 编码了一条消息: {}", ctx.channel().id(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(buf)));
        }
    }

}
