package com.platform.cac.tcp.message;



import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.cac.tcp.message.handler.IRemoteMessageHandler;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.util.ByteUtils;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Author shiyi
 * @Date 2023/12/27
 * @Description 根据数据中的帧头和指令类型，获取对应的消息处理器
 */

@ChannelHandler.Sharable
@Component
@Slf4j
public class RemoteMessageDispatcher extends SimpleChannelInboundHandler<RemoteTcpBaseDataFrame> {

    @Autowired
    private RemoteMessageHandlerContainer remoteMessageHandlerContainer;


    @Override
    public void channelRead0(ChannelHandlerContext ctx, RemoteTcpBaseDataFrame message) throws InterruptedException {
        RemoteFrameEnum messageType = getNettyMessageType(message);
        // 获得 type 对应的 MessageHandler 处理器
        IRemoteMessageHandler messageHandler = remoteMessageHandlerContainer.getMessageHandler(messageType);
        // 执行逻辑
        if (messageHandler != null) {
            log.debug("执行逻辑 {}", messageHandler.getFrameType());
            messageHandler.execute(ctx.channel(), message);
        } else {
            if (messageType != null) {
                ctx.fireChannelRead(message);
            }
        }
    }

    private RemoteFrameEnum getNettyMessageType(RemoteTcpBaseDataFrame message) {
        RemoteFrameEnum remoteFrameEnum = RemoteFrameEnum.getByCode(message.getType());
        if (remoteFrameEnum == null) {
            log.warn("未知的远程消息类型: {}, 无对应处理逻辑", ByteUtils.byteToHex(message.getType()));
            return null;
        }
        if (remoteFrameEnum.equals(RemoteFrameEnum.HEART_BEAT_RESPONSE) || remoteFrameEnum.equals(RemoteFrameEnum.HEART_BEAT_REQUEST)) {
            // 过滤心跳信息
            return null;
        }
        return remoteFrameEnum;
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.error("exceptionCaught: {}", cause.getMessage(), cause);
    }
}
