package com.platform.cac.tcp.message;



import com.platform.cac.tcp.message.handler.IRemoteMessageHandler;
import com.platform.info.enums.RemoteFrameEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author sunjipeng
 * @Date 2023/6/14
 * @Description 描述
 */
@Slf4j
@Component
public class RemoteMessageHandlerContainer implements InitializingBean {

    /**
     * 消息类型与 IGcsMessageHandler 的映射
     */
    private final Map<RemoteFrameEnum, IRemoteMessageHandler> handlers = new HashMap<>();
    @Autowired
    private ApplicationContext applicationContext;

    @Override
    public void afterPropertiesSet() throws Exception {
        // 通过 ApplicationContext 获得所有 MessageHandler Bean
        applicationContext.getBeansOfType(IRemoteMessageHandler.class).values()
                .forEach(messageHandler -> handlers.put(messageHandler.getFrameType(), messageHandler));
        log.info("[afterPropertiesSet] IRemoteMessageHandler 数量：{}", handlers.size());
    }

    /**
     * 获得类型对应的 IMessageHandler
     */
    public IRemoteMessageHandler getMessageHandler(RemoteFrameEnum remoteFrameEnum) {
        return handlers.get(remoteFrameEnum);
    }

}
