package com.platform.cac.tcp.message.handler;


import com.platform.cac.CacHpApi;
import com.platform.cac.GcsService;
import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.cac.tcp.message.dataframe.receive.CacUavCommandMessage;
import com.platform.info.GlobalData;
import com.platform.info.dto.CommandResultParam;
import com.platform.info.dto.QueryResultParam;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.info.enums.UavTypeEnum;
import com.platform.info.mapping.HaborUavMap;
import com.platform.info.mapping.UavIdMap;
import com.platform.service.clientmanage.ClientManager;
import com.platform.service.clientmanage.HaborClient;
import com.platform.util.ByteUtils;
import com.platform.util.JSONUtils;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author : shiyi
 * @Date : 2024/1/9 11:28
 * @Description : 远程中心指控指令下发
 */
@Component
@Slf4j
public class CacUavCommandIssuedHandler implements IRemoteMessageHandler {
    @Resource
    GcsService gcsService;

    @Override
    public void execute(Channel channel, RemoteTcpBaseDataFrame remoteTcpBaseDataFrame) {

        CacUavCommandMessage uavCommand = new CacUavCommandMessage(remoteTcpBaseDataFrame);
        String uavId = uavCommand.getUavId();
        UavTypeEnum uavType = UavIdMap.getUavType(uavId);
        String haborSn = HaborUavMap.getHaborSnByUavId(uavId);
        String commandUniId = uavCommand.getCommandUniId();
        byte[] uavCommandCodeId = uavCommand.getUavCommandCodeId();
        byte code = uavCommandCodeId[0];


        if (!gcsService.isSignIn()) {
            log.warn("[cac] 当前服务未连接中心指控无法下发控制指令");
            notifyControlCommandResult(uavId, commandUniId, uavType, code);
            return;
        }
        if (haborSn == null) {
            log.error("[cac] 中心指控控制指令下发失败，未找到UavId={}对应的哈勃编号", uavId);
            notifyControlCommandResult(uavId, commandUniId, uavType, code);
            return;
        }
        HaborClient client = (HaborClient) ClientManager.getClient(haborSn);
        if (client == null) {
            log.error("[cac] 中心指控参数装订指令下发失败，哈勃编号={}对应的客户端未上线", haborSn);
            notifyControlCommandResult(uavId, commandUniId, uavType, code);
            return;
        }
        if (uavCommandCodeId.length != 1) {
            log.error("[cac] 中心指控控制指令下发失败，指令码长度错误：{}", ByteUtils.bytes2HexString(uavCommandCodeId));
            notifyControlCommandResult(uavId, commandUniId, uavType, code);
            return;
        }

        log.info("[cac] 中心指控下发控制指令...{}", uavCommand);
        client.writeControlCommand(code, commandUniId);

    }

    private void notifyControlCommandResult(String uavId, String commandUniId, UavTypeEnum uavType, byte code) {
        CommandResultParam commandResultParam = new CommandResultParam();
        try {
            commandResultParam.setGcsId(GlobalData.GCS_ID);
            commandResultParam.setUavId(uavId);
            commandResultParam.setCommandUniId(commandUniId);
            commandResultParam.setCommandSource(1);
            commandResultParam.setCommandCode(ByteUtils.byteToInt(code));
            commandResultParam.setCommandResult(false?0:1);
            CacHpApi.commandNotify(commandResultParam);
        } catch (IOException e) {
            log.error("[cac] 参数装订回报通知失败：{}", commandResultParam, e);
        }
    }

    @Override
    public RemoteFrameEnum getFrameType() {
        return RemoteFrameEnum.UAV_COMMAND_ISSUED;
    }
}
