package com.platform.cac.tcp.message.handler;


import com.platform.cac.CacHpApi;
import com.platform.cac.GcsService;
import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.cac.tcp.message.dataframe.receive.CacUavParamBindMessage;
import com.platform.info.GlobalData;
import com.platform.info.dto.BindResultParam;
import com.platform.info.dto.QueryResultParam;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.info.enums.UavTypeEnum;
import com.platform.info.mapping.HaborUavMap;
import com.platform.info.mapping.UavIdMap;
import com.platform.service.clientmanage.ClientManager;
import com.platform.service.clientmanage.HaborClient;
import com.platform.util.ByteUtils;
import com.platform.util.CRCUtil;
import com.platform.util.JSONUtils;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author : shiyi
 * @Date : 2025/1/21 10:38
 * @Description : 下发参数装订指令
 */
@Component
@Slf4j
public class CacUavParamBindHandler implements IRemoteMessageHandler {
    @Resource
    GcsService gcsService;

    @Override
    public void execute(Channel channel, RemoteTcpBaseDataFrame remoteTcpBaseDataFrame) {

        CacUavParamBindMessage uavCommand = new CacUavParamBindMessage(remoteTcpBaseDataFrame);
        String uavId = uavCommand.getUavId();
        UavTypeEnum uavType = UavIdMap.getUavType(uavId);
        String haborSn = HaborUavMap.getHaborSnByUavId(uavId);
        byte commandCode = uavCommand.getCommandCode();
        byte[] commandContent = uavCommand.getCommandContent();
        String commandUniId = uavCommand.getCommandUniId();

        if (!gcsService.isSignIn()) {
            log.warn("[cac] 当前服务未连接中心指控无法下发参数装订指令");
            notifyParamBindCommandResult(uavId, commandUniId, uavType, commandCode);
            return;
        }
        if (haborSn == null) {
            log.error("[cac] 中心指控参数装订指令下发失败，未找到UavId={}对应的哈勃编号", uavId);
            notifyParamBindCommandResult(uavId, commandUniId, uavType, commandCode);
            return;
        }
        HaborClient client = (HaborClient) ClientManager.getClient(haborSn);
        if (client == null) {
            log.error("[cac] 中心指控参数装订指令下发失败，哈勃编号={}对应的客户端未上线", haborSn);
            notifyParamBindCommandResult(uavId, commandUniId, uavType, commandCode);
            return;
        }

        byte[] command14_29 = new byte[16];
        command14_29[0] = commandCode;
        System.arraycopy(commandContent, 0, command14_29, 1, 15);
        if (!Arrays.equals(uavCommand.getCrc16(), CRCUtil.getCRCfromGcsTelemetryData(command14_29, 0, 16))) {
            log.error("[cac] 中心指控参数装订指令下发失败，CRC校验失败");
            notifyParamBindCommandResult(uavId, commandUniId, uavType, commandCode);
            return;
        }
        String uavControlUniId = uavCommand.getCommandUniId();
        log.info("[cac] 中心指控下发参数装订指令...{}", uavCommand);
        client.writeParamBindCommand(commandCode, command14_29, uavControlUniId);

    }

    private void notifyParamBindCommandResult(String uavId, String commandUniId, UavTypeEnum uavType, byte code) {
        BindResultParam bindResultParam = new BindResultParam();
        try {
            bindResultParam.setGcsId(GlobalData.GCS_ID);
            bindResultParam.setUavId(uavId);
            bindResultParam.setCommandUniId(commandUniId);
            bindResultParam.setCommandSource(1);
            bindResultParam.setUavType(uavType.getRemoteCode());
            bindResultParam.setUavParamBindCode(ByteUtils.byteToInt(code));
            bindResultParam.setBindResult(false);
            CacHpApi.uavParamBindResultNotify(bindResultParam);
        } catch (IOException e) {
            log.error("[cac] 参数装订回报通知失败：{}", bindResultParam, e);
        }
    }
    @Override
    public RemoteFrameEnum getFrameType() {
        return RemoteFrameEnum.UAV_PARAM_BIND;
    }
}
