package com.platform.cac.tcp.message.handler;


import com.platform.cac.CacHpApi;
import com.platform.cac.GcsService;
import com.platform.cac.tcp.message.dataframe.RemoteTcpBaseDataFrame;
import com.platform.cac.tcp.message.dataframe.receive.CacUavParamBindMessage;
import com.platform.cac.tcp.message.dataframe.receive.CacUavParamQueryMessage;
import com.platform.info.GlobalData;
import com.platform.info.dto.QueryResultParam;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.info.enums.UavTypeEnum;
import com.platform.info.mapping.HaborUavMap;
import com.platform.info.mapping.UavIdMap;
import com.platform.service.clientmanage.ClientManager;
import com.platform.service.clientmanage.HaborClient;
import com.platform.util.ByteUtils;
import com.platform.util.CRCUtil;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Arrays;

/**
 * @Author : shiyi
 * @Date : 2025/1/21 10:38
 * @Description : 下发参数查询指令
 */
@Component
@Slf4j
public class CacUavParamQueryHandler implements IRemoteMessageHandler {
    @Resource
    GcsService gcsService;

    @Override
    public void execute(Channel channel, RemoteTcpBaseDataFrame remoteTcpBaseDataFrame) {

        CacUavParamQueryMessage uavCommand = new CacUavParamQueryMessage(remoteTcpBaseDataFrame);
        String uavId = uavCommand.getUavId();
        UavTypeEnum uavType = UavIdMap.getUavType(uavId);
        String haborSn = HaborUavMap.getHaborSnByUavId(uavId);
        String commandUniId = uavCommand.getCommandUniId();
        byte code = uavCommand.getCommandCode();

        if (!gcsService.isSignIn()) {
            log.warn("[cac] 当前服务未连接中心指控无法下发参数查询指令");
            notifyParamQueryCommandFailed(uavId, commandUniId, uavType, code);
            return;
        }
        if (haborSn == null) {
            log.error("[cac] 中心指控参数查询指令下发失败，未找到UavId={}对应的哈勃编号", uavId);
            notifyParamQueryCommandFailed(uavId, commandUniId, uavType, code);
            return;
        }
        HaborClient client = (HaborClient) ClientManager.getClient(haborSn);
        if (client == null) {
            log.error("[cac] 中心指控参数装订指令下发失败，哈勃编号={}对应的客户端未上线", haborSn);
            notifyParamQueryCommandFailed(uavId, commandUniId, uavType, code);
            return;
        }

        byte commandCode = uavCommand.getCommandCode();
        byte[] commandContent = uavCommand.getCommandContent();
        byte[] command14_29 = new byte[16];
        command14_29[0] = commandCode;
        System.arraycopy(commandContent, 0, command14_29, 1, 15);
        if (!Arrays.equals(uavCommand.getCrc16(), CRCUtil.getCRCfromGcsTelemetryData(command14_29, 0, 16))) {
            log.error("[cac] 中心指控参数查询指令下发失败，CRC校验失败");
            notifyParamQueryCommandFailed(uavId, commandUniId, uavType, code);
            return;
        }

        log.info("[cac] 中心指控下发参数查询指令...{}", uavCommand);
        client.writeParamQueryCommand(commandCode, command14_29, commandUniId);

    }

    private void notifyParamQueryCommandFailed(String uavId, String commandUniId, UavTypeEnum uavType, byte code) {
        QueryResultParam queryResultParam = new QueryResultParam();
        try {
            queryResultParam.setGcsId(GlobalData.GCS_ID);
            queryResultParam.setUavId(uavId);
            queryResultParam.setCommandUniId(commandUniId);
            queryResultParam.setCommandSource(1);
            queryResultParam.setUavType(uavType.getRemoteCode());
            queryResultParam.setUavParamQueryCode(ByteUtils.byteToInt(code));
            queryResultParam.setQueryResult(false);
            CacHpApi.uavParamQueryResultNotify(queryResultParam);
        } catch (IOException e) {
            log.error("[cac] 参数装订回报通知失败：{}", queryResultParam, e);
        }
    }

    @Override
    public RemoteFrameEnum getFrameType() {
        return RemoteFrameEnum.UAV_PARAM_QUERY;
    }
}
