package com.platform.controller;

import com.platform.model.ManyToMany;
import com.platform.model.OneToMany;
import com.platform.util.SessionCache;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 哈勃服务器 地面站-哈勃映射配置接口
 */
@RestController
@Slf4j
public class ClientController {
	@Autowired
	private SessionCache sessionCache;

	/**
	 * 获取已经连接上的session ip
	 * @return
	 */
	@GetMapping("/session/ips")
	public List<String> getSessions() {
		List<String> ips = new ArrayList<>();
		Set set= sessionCache.tcpSession.keySet();
		Iterator iterator = set.iterator();
		while (iterator.hasNext()){
			ips.add(iterator.next().toString());
		}
		return ips;
	}

	/**
	 * 添加一对一连接
	 *
	 * @param source
	 * @param target
	 * @return
	 */
	@PostMapping("/mapping/oneToOne")
	public Boolean addSnMap(@RequestParam("source") String source, @RequestParam("target") String target) {
		sessionCache.snToSnMap.put(source, target);
		sessionCache.snToSnMap.put(target, source);
		return true;
	}

	/**
	 * 添加一对多连接
	 * @param param
	 * @return
	 */
	@PostMapping("/mapping/oneToMany")
	public Boolean addSnsMap(@RequestBody OneToMany param) {
		List<String> list = param.getTarget();
		List<String> target = list.stream().map(String::toUpperCase).collect(Collectors.toList());
		String source = param.getSource().toUpperCase();
		target.forEach(sn -> sessionCache.snToSnMap.put(sn, source));
		if(!sessionCache.snList.contains(source)) {
			sessionCache.snList.add(source);
		}
		List<String> oldTarget = sessionCache.snToListMap.get(source);
		if (!ObjectUtils.isEmpty(oldTarget)) {
			oldTarget.forEach(sn ->{
				if (!target.contains(sn)) {
					target.add(sn);
				}
			});
		}
		sessionCache.snToListMap.put(source, target);
		return true;
	}

	/**
	 * 添加一对一、一对多、多对多映射
	 * @param param
	 * @return
	 */
	@PostMapping("/mapping/all")
	public Boolean addSourcesAndTargetsMap(@RequestBody ManyToMany param) {
		List<String> targetList = param.getTarget().stream().map(String::toUpperCase).collect(Collectors.toList());
		List<String> sourceList = param.getSource().stream().map(String::toUpperCase).collect(Collectors.toList());
		List<String> sourceTmp = new ArrayList<>();
		targetList.forEach(target->{
			List<String> oldSource = sessionCache.mappingListMap.get(target);
			if (!ObjectUtils.isEmpty(oldSource)) {
				oldSource.forEach(sn ->{
					if (!sourceList.contains(sn)) {
						sourceList.add(sn);
					}
				});
			}
			List<String> newSource= new ArrayList<>();
			newSource.addAll(sourceList);
			if (!sourceTmp.isEmpty()){
				newSource.addAll(sourceTmp);
				sourceTmp.clear();
			}
			sessionCache.mappingListMap.put(target, newSource);
		});
		List<String> targetTmp = new ArrayList<>();
 		sourceList.forEach(source ->{
			List<String> oldTarget = sessionCache.mappingListMap.get(source);
			if (!ObjectUtils.isEmpty(oldTarget)) {
				oldTarget.forEach(sn ->{
					if (!targetList.contains(sn)) {
						targetTmp.add(sn);
					}
				});
			}
			List<String> newTarget = new ArrayList<>();
			newTarget.addAll(targetList);
			if (!targetTmp.isEmpty()){
				newTarget.addAll(targetTmp);
				targetTmp.clear();
			}
			sessionCache.mappingListMap.put(source, newTarget);
		});
		
		// 更新配置文件
		sessionCache.saveMappingListMapToFile();
		return true;
	}

	/**
	 * 删除映射关系
	 * @param param
	 * @return
	 */
	@PostMapping("/mapping/delete")
	public Boolean deleteSourcesAndTargetsMap(@RequestBody ManyToMany param) {
		List<String> sourceList = param.getSource().stream().map(String::toUpperCase).collect(Collectors.toList());
		List<String> targetList = param.getTarget().stream().map(String::toUpperCase).collect(Collectors.toList());
		List<String> all = new ArrayList<>();
		all.addAll(sourceList);
		all.addAll(targetList);
		all.forEach(code ->{
			List<String> valueList = sessionCache.mappingListMap.get(code);
			List<String> deletedList = new ArrayList<>();
			if (!ObjectUtils.isEmpty(valueList)) {
				valueList.forEach(value ->{
					if (all.contains(value)){
						deletedList.add(value);
					}
				});
				deletedList.forEach(deleted -> valueList.remove(deleted));
				if (valueList.size() > 0) {
					sessionCache.mappingListMap.put(code, valueList);
				} else{
					sessionCache.mappingListMap.remove(code);
				}
			}
		});

		// 更新配置文件
		sessionCache.saveMappingListMapToFile();
		return true;
	}

	/**
	 * 获取一对一映射关系
	 * @return
	 */
	@GetMapping("/mapping/oneToOne")
	public Map<String, String> getSnMap() {
		return sessionCache.snToSnMap;
	}

	/**
	 * 获取一对多映射关系
	 * @return
	 */
	@GetMapping("/mapping/oneToMany")
	public Map<String, List<String>> getSnListMap() {
		return sessionCache.snToListMap;
	}

	/**
	 * 获取所有映射关系
	 * @return
	 */
	@GetMapping("/mapping/many")
	public Map<String, List<String>> getTargetListMap() {
		return sessionCache.mappingListMap;
	}

	/**
	 * 配置监控接口
	 *
	 * @param supervisor
	 * @param monitored
	 * @return
	 */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "supervisor", value = "监控者", required = true),
            @ApiImplicitParam(name = "monitored", value = "被监控者", required = true),
    })
	@GetMapping("/sn/mapping/monitor/add")
	@ApiModelProperty(value = "配置监控接口")
	public Boolean addMonitorMap(String supervisor, String monitored) {
        if(StringUtils.isEmpty(supervisor) || StringUtils.isEmpty(monitored)){
            return false;
        }
        sessionCache.snToMonitorMap.clear();
		sessionCache.snToMonitorMap.put(monitored,supervisor);
		return true;
	}

	/**
	 * 查询监控关系
	 * @return
	 */
	@GetMapping("/sn/mapping/monitor/get")
	@ApiModelProperty(value = "查询监控关系接口")
	public Map<String,String> getMonitorMap() {
		return sessionCache.snToMonitorMap;
	}
}
