package com.platform.info.enums;

/**
 * @Author : shiyi
 * @Date : 2024/1/2 13:46
 * @Description : 运管和中心指控帧类型枚举
 */

public enum RemoteFrameEnum {
    // 运管
    GCS_AUTH_CAC_REQUEST((byte) 0x01, "地面站接入数据路由认证请求"),
    GCS_AUTH_CAC_RESPONSE((byte)0x02, "地面站接入数据路由认证响应"),
    HEART_BEAT_REQUEST((byte) 0x03, "心跳请求"),
    HEART_BEAT_RESPONSE((byte) 0x04, "心跳响应"),
    FLIGHT_PLAN_REPLY_REQUEST((byte) 0x05, "飞行计划审批结果通知"),
    FLIGHT_PLAN_REPLY_RESPONSE((byte) 0x06, "飞行计划审批结果响应"),
    FLY_REPLY_REQUEST((byte) 0x07, "放飞申请结果通知"),
    FLY_REPLY_RESPONSE((byte) 0x08, "放飞申请结果响应"),
    RECEIVE_ALARM_REQUEST((byte) 0x09, "告警信息"),
    RECEIVE_ALARM_RESPONSE((byte) 0x0A, "告警信息接收响应"),
    RECEIVE_ATC_REQUEST( (byte) 0x0B, "管制指令通知"),
    RECEIVE_ATC_RESPONSE((byte) 0x0C, "管制指令接收响应"),
    FLIGHT_PLAN_REVOKE_REQUEST((byte) 0x0D, "飞行计划撤销通知"),
    FLIGHT_PLAN_REVOKE_RESPONSE((byte) 0x0E, "飞行计划撤销响应"),
    UAV_TELEMETRY_DATA_TRANSFER((byte) 0x0F, "无人机遥测数据透传"),

    // 中心指控
    UAV_CONTROL_APPLY((byte) 0x10, "申请无人机控制权"),
    UAV_CONTROL_REPLY((byte) 0x11, "回复无人机控制权"),
    UAV_CONTROL_RECEIVE((byte) 0x12, "接收无人机控制权"),
    UAV_COMMAND_ISSUED((byte) 0x13, "无人机控制指令下发"),
    UAV_COMMAND_QUERY((byte) 0x14, "无人机控制指令查询"),

    // 自动化机场
    AIRPORT_COMMAND_ISSUED((byte) 0x15, "机场指令下发"),
    AIRPORT_COMMAND_QUERY((byte) 0x16, "机场指令查询"),
    AIRPORT_STATUS_QUERY((byte) 0x17, "机场状态查询"),
    AIRPORT_STATE_OF_CHARGE_QUERY((byte)0x18, "电格电量查询"),
    AIRPORT_CONTAINER_STATUS_QUERY((byte)0x19, "货格状态查询"),

    AIRSPACE_REPLY_REQUEST((byte) 0x1A, "空域申请结果通知"),
    AIRSPACE_REPLY_RESPONSE((byte) 0x1B, "空域申请结果收到响应"),

    UAV_PARAM_BIND((byte) 0x2A, "无人机参数装订"),
    UAV_PARAM_QUERY((byte) 0x2B, "无人机参数查询"),
    UAV_ROUTE_BIND((byte) 0x2C, "无人机航线装订"),


    ;
    private final byte code;
    private final String info;
    public byte getCode() {
        return code;
    }

    public String getInfo() {
        return info;
    }


    RemoteFrameEnum(byte code, String info) {
        this.code = code;
        this.info = info;
    }

    public static RemoteFrameEnum getByCode(byte frameCode) {
        for (RemoteFrameEnum enums : RemoteFrameEnum.values()) {
            if (enums.code == frameCode) {
                return enums;
            }
        }
        return null;
    }

}
