package com.platform.info.enums;

import lombok.Getter;

/**
 * @Author : shiyi
 * @Date : 2024/1/9 9:27
 * @Description : 无人机类型枚举
 */
@Getter
public enum UavTypeEnum {
    //
    FP981A (0x01, 1,"981A", false),
    FP981C (0x02, 2,"981C", false),
    FP98 (0x04, 4,"98", true),
    FP985 (0x08, 8,"985", true),
    FP981CS (0x10, 16,"981CS", true),
    FH981CH (0x20, 32,"981CH", true),
    // 此后的编码和中心指控编码区分开
    FH981CH_HYBRID (0x21, 64,"981CH-二代机混动", false),
    FH981CH_ELECTRIC (0x22, 128,"981CH-二代机电动", false),
    ;
    private final int gcsCode;
    private final int remoteCode;
    private final String info;
    private final boolean onlyMonitor; // 仅具有监视功能，但不控制

    UavTypeEnum(int code, int remoteCode, String info, boolean onlyMonitor) {
        this.gcsCode = code;
        this.remoteCode = remoteCode;
        this.info = info;
        this.onlyMonitor = onlyMonitor;
    }

    public int getGcsCode() {
        return gcsCode;
    }
    public int getRemoteCode() {
        return remoteCode;
    }
    public String getInfo() {
        return info;
    }
    public boolean isOnlyMonitor() {
        return onlyMonitor;
    }

    public String toString() {
        return this.info;
    }
    public static UavTypeEnum getByGcsCode(int code) {
        for (UavTypeEnum typeEnum : UavTypeEnum.values()) {
            if (typeEnum.getGcsCode() == code) {
                return typeEnum;
            }
        }
        return null;
    }
    public static UavTypeEnum getByRemoteCode(int code) {
        for (UavTypeEnum typeEnum : UavTypeEnum.values()) {
            if (typeEnum.getRemoteCode() == code) {
                return typeEnum;
            }
        }
        return null;
    }
}
