package com.platform.info.mapping;



import com.platform.info.enums.UavTypeEnum;

import java.util.concurrent.ConcurrentHashMap;

/**当前在控无人机的缓存表，每次查询在控无人机的时候将更新该表
 * Note: 该列表不保证和后端同步，如需准确获知在控情况请直接调用查询接口
 */

public class UavIdControlMap {
    // 所有型号，uavId到fkId的映射
    private static final ConcurrentHashMap<String, Integer> globalMapId = new ConcurrentHashMap<>();

    // 各个型号，fkId到uavId的映射
    private static final ConcurrentHashMap<Integer, String> uavMapId981A = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer, String> uavMapId981C = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer, String> uavMapId98 = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer, String> uavMapId985 = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer, String> uavMapId981CS = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<UavTypeEnum, ConcurrentHashMap<Integer, String>> uavTypeMapId = new ConcurrentHashMap<>();


    static {
        uavTypeMapId.put(UavTypeEnum.FP981A, uavMapId981A);
        uavTypeMapId.put(UavTypeEnum.FP981C, uavMapId981C);
        uavTypeMapId.put(UavTypeEnum.FP98, uavMapId98);
        uavTypeMapId.put(UavTypeEnum.FP985, uavMapId985);
        uavTypeMapId.put(UavTypeEnum.FP981CS, uavMapId981CS);
    }
    // private static BidiMap<Integer, String> uavMapId = new DualHashBidiMap<>();


    /**根据飞机型号增加映射关系*/
    public static void addMap(UavTypeEnum uavType, Integer fkId, String uavId){
        globalMapId.put(uavId, fkId);
        if (uavType != null && uavTypeMapId.containsKey(uavType)) {
            uavTypeMapId.get(uavType).put(fkId, uavId);
        }
    }


    /** 获取指定型号的映射关系*/
    public static String showMap(UavTypeEnum uavType){
        if (uavType != null && uavTypeMapId.containsKey(uavType)) {
            return uavTypeMapId.get(uavType).toString();
        }
        return null;
    }

    /**
     * 获取所有型号的映射关系
     */
    public static String showAllMap(){
        StringBuilder sb = new StringBuilder();
        sb.append("{\n");
        uavTypeMapId.forEach((uavType, uavIdMap) -> {
            sb.append(uavType.getInfo()).append(":").append(uavTypeMapId.get(uavType)).append("\n");
        });
        sb.append("}");
        return sb.toString();
    }



    /**根据指定型号的fkId获取uavId*/
    public static String getUavId(UavTypeEnum uavType, Integer fkId){
        ConcurrentHashMap<Integer, String> uavMapId = uavTypeMapId.get(uavType);
        if (fkId ==null || uavMapId == null||  !uavMapId.containsKey(fkId)){
            throw new IllegalArgumentException(uavType.getInfo() + "类型fkId=" + fkId + " 无法转换为uavId， 该类型映射关系: " + uavMapId);
        }
        return uavMapId.get(fkId);
    }


    /**
     * 根据uavId获取飞控id
     */
    public static int getFkId(String uavId){
        if (uavId ==null || !globalMapId.containsKey(uavId)){
            throw new IllegalArgumentException("uavId=" + uavId + " not found in uavMapId: " + globalMapId);
        }
        return globalMapId.get(uavId);
    }

    /** uavId是否是当前地面站可控 */
    public static boolean uavIdControlling(UavTypeEnum uavType, String uavId){
        return uavTypeMapId.get(uavType).containsValue(uavId);
    }

    /** fkId是否是当前地面站可控 */
    public static boolean fkIdControlling(UavTypeEnum uavType, Integer fkId){
        return uavTypeMapId.get(uavType).containsKey(fkId);
    }
    public static void clear() {
        uavTypeMapId.forEach((uavType, uavIdMap) -> uavIdMap.clear());
    }
}
