package com.platform.service;

import com.platform.config.ServiceConfig;
import com.platform.service.handler.CacDataRouterHandler;
import com.platform.service.handler.InMessageHandler1;
import com.platform.service.handler.TelemetryDecoder;
import com.platform.util.SessionCache;
import com.platform.util.StringUtil;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.bytes.ByteArrayEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import io.netty.util.ResourceLeakDetector;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class ServerService {

    public ChannelFuture mChannelFuture;

    @Autowired
    private ServiceConfig config;
    @Autowired
    private StringUtil stringUtil;
    @Autowired
    private SessionCache sessionCache;

    @Autowired
    InMessageHandler1 inMessageHandler;
    @Autowired
    CacDataRouterHandler cacDataRouterHandler;
    private void startServer() {
        //服务端需要2个线程组  boss处理客户端连接  work进行客服端连接之后的处理
        log.warn("【读取配置端口】 端口 = " + config.getPort());
        EventLoopGroup boss = new NioEventLoopGroup();
        EventLoopGroup work = new NioEventLoopGroup();
        try {
            ServerBootstrap bootstrap = new ServerBootstrap();
            //服务器 配置
            bootstrap.group(boss, work);
            bootstrap.channel(NioServerSocketChannel.class);
            bootstrap.childHandler(new ChannelInitializer<Channel>() {
                @Override
                protected void initChannel(Channel channel) throws Exception {
                    channel.pipeline().addLast(new ByteArrayEncoder());
                    if (!config.isDebug()) {
                        channel.pipeline().addLast(new IdleStateHandler(0, 0,
                                5, TimeUnit.MINUTES));
                    }
                    channel.pipeline()
                            .addLast("OriginalHandler", inMessageHandler)  // 原始哈勃消息解码器
                            .addLast("TelemetryDecoder", new TelemetryDecoder()) // eb90数据分离
                            .addLast("CacDataRouter", cacDataRouterHandler); // cac数据转发
                }
            });
           ResourceLeakDetector.setLevel(ResourceLeakDetector.Level.PARANOID); // 内存泄露检测，上线记得关掉
            bootstrap.option(ChannelOption.SO_BACKLOG, 2048);
            bootstrap.childOption(ChannelOption.SO_KEEPALIVE, true);
            bootstrap.childOption(ChannelOption.TCP_NODELAY, true);
            mChannelFuture = bootstrap.bind(config.getPort()).sync();
            mChannelFuture.channel().closeFuture().sync();
            log.warn("【服务器启动成功========端口：" + config.getPort() + "】");
        } catch (Exception e) {
            log.error("【服务器启动失败】", e);
        } finally {
            //关闭资源
            boss.shutdownGracefully();
            work.shutdownGracefully();
        }
    }

    @PostConstruct()
    public void init() {
        log.warn("【初始化......】");
        //需要开启一个新的线程来执行netty server 服务器
        new Thread(() -> startServer()).start();
    }
}
