package com.platform.service.clientmanage;


import com.platform.info.enums.ClientTypeEnum;
import io.netty.channel.Channel;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.net.InetSocketAddress;
import java.util.Objects;



/**
 * @Author : shiyi
 * @Date : 2023/12/18 14:51
 * @Description : 客户端类
 */
@Slf4j @Getter
public abstract class BaseClient {
    protected Channel channel;
    protected String sn; // 客户端唯一编号

    protected String ip;
    protected String port;
    protected String serverPort;
    protected volatile boolean online = false;
    // protected boolean online = false;
    public BaseClient(String sn, Channel channel) {
        this.sn = sn;
        this.channel = channel;
        InetSocketAddress remoteAddress = (InetSocketAddress) channel.remoteAddress();
        InetSocketAddress localAddress = (InetSocketAddress) channel.localAddress();
        ip = remoteAddress.getHostString();
        port = String.valueOf(remoteAddress.getPort());
        serverPort = String.valueOf(localAddress.getPort());
    }

    /**
     * 报告客户端上线
     */
    public abstract boolean online();

    /**
     * 报告客户端下线， 仅当channel关闭时调用
     */
    public abstract boolean offline();

    /**获取当前client的在线状态，表示是否进行与远程服务的通信*/
    public abstract boolean isOnline();
    // public ChannelFuture sendMessage(byte frameType, byte[] data) {
    //     return ClientManager.send(getClientType(), channel, frameType, data);
    // }

    public abstract ClientTypeEnum getClientType();
    @Override
    public String toString() {
        return getClass().getSimpleName()+ "{" +
                "channel=" + channel.id() +
                ", sn='" + sn + '\'' +
                ", ip='" + ip + '\'' +
                ", port=" + port +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        BaseClient that = (BaseClient) o;
        return Objects.equals(sn, that.sn);
    }

    @Override
    public int hashCode() {
        return Objects.hash(sn);
    }
}
