package com.platform.service.clientmanage;


import com.platform.info.enums.CommandTypeEnum;
import io.netty.buffer.ByteBuf;
import lombok.extern.slf4j.Slf4j;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

/** 记录一组指令执行状态（主要是航线装订），回报指令结果*/
@Slf4j
public class CommandQueueRecord extends CommandRecord {
    protected Queue<byte[]> commandQueue;
    public static final int WAITING_NEXT = 2; // 等待下一个节点回报

    public CommandQueueRecord(CommandTypeEnum commandType, HaborClient client, int contentSize){
        super(commandType, client, contentSize);
    }
    public void initCommandQueue(List<byte[]> commandList){
        this.commandQueue = new LinkedList<>(commandList);
        this.commandBuf.writerIndex(0);
        this.commandBuf.writeBytes(commandQueue.peek());
    }

    /**
     * 等待判断下一个指令
     *
     * @return
     */
    public boolean completeOrWaitNext() {
        this.commandQueue.poll();
        if (commandQueue.isEmpty()) {
            this.status = NO_COMMAND;
            return true;
        }
        this.commandBuf.writeBytes(commandQueue.peek());
        return false;
    }

    /** 设置指令状态为等待下一个节点*/
    @Override
    public void recordCommand(byte commandCode, String uniId, ByteBuf commandSlice) {
        super.recordCommand(commandCode, uniId, commandSlice);
        this.status = WAITING_NEXT; // 覆盖父类状态
    }

    @Override
    public void clearCommand() {
        cancelScheduleTimeout();
        this.commandBuf.clear();
        this.commandUniId = null;
        this.commandQueue = null;
        this.status = NO_COMMAND;
    }

    public void waitNext() {
        cancelScheduleTimeout();
    }

}
