package com.platform.service.clientmanage;


import com.platform.info.GlobalData;
import com.platform.info.enums.CommandTypeEnum;
import com.platform.info.enums.RemoteFrameEnum;
import com.platform.info.enums.UavTypeEnum;
import com.platform.info.mapping.HaborUavMap;
import com.platform.info.mapping.UavIdMap;
import com.platform.util.ByteUtils;
import com.platform.util.JSONUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.Channel;
import jdk.nashorn.internal.runtime.regexp.joni.exception.ValueException;
import lombok.Data;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.List;

/**
 * @Author : shiyi
 * @Date : 2024/9/18 10:38
 * @Description : 飞控客户端-981A
 */
@Slf4j @Getter
public class HaborClient981A extends HaborClient {

    private final CommandManager commandManager = CommandManager.getInstance();
    private static final byte[] UPPER_DATA_3_4 = new byte[]{0x02, 0x01};

    /**常发数据内容，被填入ByteBuf初始内容，所有飞机共用, 禁止修改*/
    private static final byte C0 = 0x00; // 空白数据自定义特征字
    private static final byte[] COMMON_DATA_14_29 = new byte[]{
            0x21, C0, C0, C0, C0, // 14~29
            C0, C0, C0, C0, C0,
            C0, C0, C0, C0, C0, C0,
    };

    /**控制帧数据内容，被填入ByteBuf初始内容，*/
    private static final byte C1 = 0x00; // 空白数据自定义特征字
    private static final byte[] CONTROL_DATA_14_29 = new byte[]{
            0x21, C1, C1, C1, C1, // 14~29
            C1, C1, C1, C1, C1,
            C1, C1, C1, C1, C1, C1,
    };

    // /**装订帧数据内容*/
    // private static final byte C2 = 0x03; // 空白数据自定义特征字
    // private static final byte[] BIND_DATA_7_30 = new byte[]{
    //         0x00, 0x00, 0x00, // 飞行开关指令7~9, 初始化或无指令3个0x00
    //         0x00, 0x00, 0x00, 0x00, // 飞行连续指令，中心指控不管，10~13
    //         C2, C2, C2, C2, C2, // 14~29, 由中心指控填充
    //         C2, C2, C2, C2, C2,
    //         C2, C2, C2, C2, C2, C2,
    //         0x00 // 30
    // };
    //
    // /**查询帧数据内容*/
    // private static final byte C3 = 0x04; // 空白数据自定义特征字
    // private static final byte[] QUERY_DATA_7_30 = new byte[]{
    //         0x00, 0x00, 0x00, // 飞行开关指令7~9, 初始化或无指令3个0x00
    //         0x00, 0x00, 0x00, 0x00, // 飞行连续指令，中心指控不管，10~13
    //         C2, C2, C2, C2, C2, // 14~29, 由中心指控填充
    //         C2, C2, C2, C2, C2,
    //         C2, C2, C2, C2, C2, C2,
    //         0x00 // 30
    // };
    //
    // /**航线帧数据内容*/
    // private static final byte C4 = 0x05; // 空白数据自定义特征字
    // private static final byte[] BIND_DATA_7_30 = new byte[]{
    //         0x00, 0x00, 0x00, // 飞行开关指令7~9, 初始化或无指令3个0x00
    //         0x00, 0x00, 0x00, 0x00, // 飞行连续指令，中心指控不管，10~13
    //         C2, C2, C2, C2, C2, // 14~29, 由中心指控填充
    //         C2, C2, C2, C2, C2,
    //         C2, C2, C2, C2, C2, C2,
    //         0x00 // 30
    // };
    /**控制指令帧*/
    public HaborClient981A(String haborSn, Channel channel) {
        super(haborSn, channel);
        uavType = UavTypeEnum.FP981A;
        uavId = HaborUavMap.getUavIdByHaborSn(haborSn);
        uavType = UavIdMap.getUavType(uavId);
        fkId = (byte) UavIdMap.getFkId(uavId);
        commonDataFreq = DEFAULT_COMMON_DATA_FREQ; // 发送频率, 每秒发送帧数
        targetPeriod = 1000 / commonDataFreq;
    }

    @Override
    protected void resourceAllocate() {
        if (resourceAllocated) {
            return;
        }
        // 常发数据帧初始化
        commonDataBuf = new CommonDataBuf(FRAME_SIZE);
        initByteBuf(commonDataBuf);
        commonDataBuf.buf.writerIndex(13).writeBytes(COMMON_DATA_14_29); // 写入7-30字节

        // 开关指令数据帧初始化
        controlDataBuf = new ControlDataBuf(FRAME_SIZE);
        initByteBuf(controlDataBuf);
        controlDataBuf.buf.writerIndex(13).writeBytes(CONTROL_DATA_14_29);

        // 参数装订指令数据帧初始化
        bindDataBuf = new BindDataBuf(FRAME_SIZE);
        initByteBuf(bindDataBuf);

        // 参数查询指令数据帧初始化
        queryDataBuf = new QueryDataBuf(FRAME_SIZE);
        initByteBuf(queryDataBuf);

        // 航线装订指令数据帧初始化
        routeDataBuf = new RouteDataBuf(FRAME_SIZE);
        initByteBuf(routeDataBuf);

        dataBufToSend = commonDataBuf; // note 默认发送常发数据帧

        if (GlobalData.GCS_ID == null) {
            throw new ValueException("GCS_ID is null");
        }
        if (GlobalData.AUTHORIZATION == null) {
            throw new ValueException("AUTHORIZATION is null");
        }
        // 中心指控数据帧初始化
        cacHeadLength = 5 // 魔术2 + 版本号1 + 序列化算法1 + 帧类型1
                + (GlobalData.GCS_ID.length()+1)
                + (GlobalData.AUTHORIZATION.length()+1)
                + (uavId.length()+1)
                + 2 + 4; //包序号 + 遥测数据长度
        telemetryBufHeadToCac = PooledByteBufAllocator.DEFAULT.buffer(cacHeadLength);
        telemetryBufHeadToCac.writeShort(GlobalData.REMOTE_HEAD);
        telemetryBufHeadToCac.writeByte(GlobalData.REMOTE_VERSION2);
        telemetryBufHeadToCac.writeByte(GlobalData.REMOTE_SER_ALG);
        telemetryBufHeadToCac.writeByte(RemoteFrameEnum.UAV_TELEMETRY_DATA_TRANSFER.getCode());
        telemetryBufHeadToCac.writeByte((byte) GlobalData.GCS_ID.length());
        telemetryBufHeadToCac.writeBytes(GlobalData.GCS_ID.getBytes());
        telemetryBufHeadToCac.writeByte((byte) GlobalData.AUTHORIZATION.length());
        telemetryBufHeadToCac.writeBytes(GlobalData.AUTHORIZATION.getBytes());
        telemetryBufHeadToCac.writeByte((byte) uavId.length());
        telemetryBufHeadToCac.writeBytes(uavId.getBytes());
        telemetryBufHeadToCac.writeShort(0);
        telemetryBufHeadToCac.writeInt(FRAME_SIZE);

        previousControl = new CommandRecord(CommandTypeEnum.CONTROL,this,16);
        previousParamBind = new CommandRecord(CommandTypeEnum.PARAM_BIND, this, 16);
        previousParamQuery = new CommandRecord(CommandTypeEnum.PARAM_QUERY, this, 16);
        previousRouteBind = new CommandQueueRecord(CommandTypeEnum.ROUTE_BIND, this, 16);

        resourceAllocated = true;
    }

    /**初始化发送帧内容*/
    private void initByteBuf(DataByteBuf dataByteBuf) {
        ByteBuf buf = dataByteBuf.buf;
        buf.writerIndex(0);
        buf.setZero(0, FRAME_SIZE); // 先全部赋0，非0数在下面修改
        buf.writeBytes(HaborClient.HEAD);
        buf.writeBytes(UPPER_DATA_3_4);
        buf.writeByte(fkId);
        // 10-13字节 默认为0x00
        // 14-29字节 在外部动态赋值

    }


    /**发送常发帧*/
    @Override
    protected void writeCommonData() {
        if (!isNeedToSendCommonData()) {
            // 如果当前处于暂停状态，则不发送数据
            return;
        }

        dataBufToSend.buf.setByte(5, frameIdx.getAndIncrement()); // 帧序号
        addCRC16(dataBufToSend.buf); // 统一添加CRC校验
        channel.writeAndFlush(dataBufToSend.buf.retain());
        dataBufToSend.addSendCount();

        if (dataBufToSend instanceof CommonDataBuf) {
            // log.debug("[cac] {} 发送常发帧: {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes((dataBufToSend.buf))));
        }
        if (dataBufToSend instanceof ControlDataBuf) {
            log.debug("[cac] {} 发送控制指令: {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(dataBufToSend.buf)));
            if (dataBufToSend.getSendCount() >= ((ControlDataBuf) dataBufToSend).needCount) {
                log.info("[cac] {} 控制指令发送完毕, {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(dataBufToSend.buf)));
                switchToCommonData();
            }
        }

        if (dataBufToSend instanceof QueryDataBuf) {
            log.debug("[cac] {} 发送查询指令: {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(dataBufToSend.buf)));
        }

        if (dataBufToSend instanceof BindDataBuf) {
            log.debug("[cac] {} 发送参数装订指令: {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(dataBufToSend.buf)));
        }

        if (dataBufToSend instanceof RouteDataBuf) {
            log.debug("[cac] {} 发送航线装订指令: {}", shortInfo(), ByteUtils.bytes2HexString(ByteBufUtil.getBytes(dataBufToSend.buf)));
        }

    }

    /**
     * 下发控制指令
     * @param commandCode 第14字节的指令码
     * @param controlUniId 指令唯一码
     * */
    @Override
    public void writeControlCommand(byte commandCode, String controlUniId) {
        controlDataBuf.buf.setByte(6, commandCode); // 控制指令
        controlDataBuf.buf.setByte(7, commandCode); // 控制指令
        controlDataBuf.buf.setByte(8, commandCode); // 控制指令
        controlDataBuf.needCount = commandNumber(commandCode); // 设置需要发送的次数
        setCurrentSendData(controlDataBuf, 20); // 设置当前发送的数据类型
        previousControl.recordCommand(commandCode, controlUniId, controlDataBuf.buf.slice(13,16)); // 记录控制指令
        // for (int i = 0; i < commandNumber(commandCode); i++) {
        //     controlDataBuf.setByte(5, frameIdx.getAndIncrement()); // 帧序号
        //     channel.writeAndFlush(controlDataBuf.retain());
        //     log.debug("{}-{}发送控制指令帧({}): {}", sn, uavType, ByteUtils.byteToHex(commandCode), ByteBufUtil.hexDump(controlDataBuf));
        // }
        // previousControl.recordCommand(commandCode, controlUniId, controlDataBuf.slice(13,16)); // 记录控制指令

    }


    /**
     * 下发参数装订指令
     * @param commandCode 第14字节的指令码
     * @param commandContent 14-29字节内容
     * @param controlUniId 指令唯一码
     */
    @Override
    public void writeParamBindCommand(byte commandCode, byte[] commandContent, String controlUniId) {
        // try {
        // byte[] test = {
        //         (byte) 0x39, (byte) 0x32, (byte) 0x00, (byte) 0x36, (byte) 0x32, (byte) 0x14, (byte) 0xF6, (byte) 0x00, (byte) 0x00, (byte) 0x28, (byte) 0x05, (byte) 0x05, (byte) 0x00, (byte) 0x00, (byte) 0x04, (byte) 0x00
        // };
        // 39 32 00 36 32 14 F6 00 00 28 05 05 00 00 04 00
        // 39 32 00 36 32 14 F6 00 00 28 05 05 00 05 00 03
        bindDataBuf.buf.setBytes(13, commandContent);
        setCurrentSendData(bindDataBuf); // 设置当前发送的数据类型
        previousParamBind.recordCommand(commandCode, controlUniId, bindDataBuf.buf.slice(13,16)); // 记录参数装订指令
        // } catch (Exception e) {
        //     log.error("{}-{}发送参数装订指令帧({})异常: {}", sn, uavType, ByteUtils.byteToHex(commandCode), e.getMessage());
        // } finally {
        //     resumeSendCommonData();
        // }
    }

    /**
     * 下发参数查询指令
     * @param commandCode 第14字节的指令码
     * @param commandContent 14-29字节内容
     * @param controlUniId 指令唯一码
     */
    @Override
    public void writeParamQueryCommand(byte commandCode, byte[] commandContent, String controlUniId) {
        queryDataBuf.buf.setBytes(13, commandContent);
        setCurrentSendData(queryDataBuf); // 设置当前发送的数据类型
        previousParamQuery.recordCommand(commandCode, controlUniId, queryDataBuf.buf.slice(13,16)); // 记录参数查询指令
    }

    /**
     * 逐个航点下发参数查询指令
     * @param commandCode 第14字节的指令码
     * @param commandContent 14-29字节内容
     * @param controlUniId 指令唯一码
     */
    private void writeRouteBindCommand0(byte commandCode, byte[] commandContent, String controlUniId) {
        routeDataBuf.buf.setBytes(13, commandContent);
        setCurrentSendData(routeDataBuf); // 设置当前发送的数据类型
        previousRouteBind.recordCommand(commandCode, controlUniId, routeDataBuf.buf.slice(13,16)); // 记录参数查询指令
    }

    /**
     * 下发参数查询指令
     * @param commandCode 第14字节的指令码
     * @param routeInfoList 14-29字节内容列表
     * @param controlUniId 指令唯一码
     */
    public void writeRouteBindCommand(byte commandCode, List<byte[]> routeInfoList, String controlUniId) {
        previousRouteBind.initCommandQueue(routeInfoList);
        writeRouteBindCommand0(commandCode, routeInfoList.get(0), controlUniId); //发送第一个节点
    }

    /**
     * 注入回报帧处理, 如果正常处理，返回true，否则返回false继续透传
     * return boolean 是否处理成功
     * */
    @Override

    protected boolean processInjectResponse(ByteBuf msg) {
        // 只处理注入回报帧A0
        if (msg.getByte(2) == (byte) 0xA0) {
            if (checkParamBindResponse(msg)) return true; // 检查参数装订回报
            if (checkParamQueryResponse(msg)) return true;  // 检查参数查询回报
            if (checkRouteBindResponse(msg)) return true; // 检查航线装订回报
        }
        if (msg.getByte(2) == (byte) 0xA1) {
            if (checkControlResponse(msg)) return true; // 检查控制指令回报
        }

        return false;
    }


    /** 检查控制指令回报*/
    private boolean checkControlResponse(ByteBuf msg) {
        if (CommandRecord.WAITING_RESULT == previousControl.getStatus()) {
            // log.debug("当前指令{}, uniId:{}, A1帧指令回报: {}", ByteUtils.byteToHex(previousControl.code), previousControl.commandUniId, ByteUtils.byteToHex(msg.getByte(29)));
            previousControl.addReceiveFrameCount();
            if (msg.getByte(29) == previousControl.code) {
                try {
                    commandManager.submitResultNotification(previousControl, true);
                    return true;
                } catch (Exception e) {
                    log.warn("[cac] {} 控制指令回报处理异常:{}",shortInfo(), e.getMessage());
                } finally {
                    switchToCommonData(); // 恢复常发数据帧
                }
            }

        }
        return false;
    }

    /** 检查参数装订回报*/
    private boolean checkParamBindResponse(ByteBuf msg) {
        if (CommandRecord.WAITING_RESULT == previousParamBind.getStatus()) {
            previousParamBind.addReceiveFrameCount();
            if (previousParamBind.commandMatch(msg.slice(13,16))) {
                try {
                    commandManager.submitResultNotification(previousParamBind, true);
                    return true;
                } finally {
                    switchToCommonData(); // 恢复常发数据帧
                }
            }
        }
        return false;
    }
    /**检查参数查询回报*/
    private boolean checkParamQueryResponse(ByteBuf msg) {
        if (CommandRecord.WAITING_RESULT == previousParamQuery.getStatus()) {
            previousParamQuery.addReceiveFrameCount();
            if (msg.getByte(13) == previousParamQuery.code) { // fixme 匹配的不是同样的内容
                try {
                    UavQueryResultParam uavQueryResultParam = null;
                    switch (msg.getByte(13)) {
                        case (byte) 0x41: // 起飞参数查询
                            uavQueryResultParam = new UavTakeOffParamQueryResultParam981A(msg.slice(13, 16));
                            break;
                        case (byte) 0x33: // 重量查询
                            uavQueryResultParam = new UavWeightQueryResultParam981A(msg.slice(13, 16));
                            break;
                        default:
                            log.warn("[cac] {} 参数查询结果回报，未知的指令类型：{}", shortInfo(), ByteUtils.byteToHex(msg.getByte(13)));
                    }
                    previousParamQuery.setUavQueryResultParam(uavQueryResultParam);
                    commandManager.submitResultNotification(previousParamQuery, true);
                    return true;
                } catch (Exception e) {
                    log.warn("[cac] {} 参数查询结果回报处理异常:{}",shortInfo(), e.getMessage());
                } finally {
                    switchToCommonData(); // 恢复常发数据帧
                }
            }
        }
        return false;
    }

    /**检查航线装订回报*/
    private boolean checkRouteBindResponse(ByteBuf msg) {
        if (CommandQueueRecord.WAITING_NEXT == previousRouteBind.getStatus()) {
            previousRouteBind.addReceiveFrameCount();
            if (previousRouteBind.commandMatch(msg.slice(13, 16))) {
                if (previousRouteBind.completeOrWaitNext()) { // 如果指令列表均已回报，则判定成功，向中心指控回报, 并恢复常发帧
                    try {
                        commandManager.submitResultNotification(previousRouteBind, true);
                        return true;
                    } finally {
                        switchToCommonData(); // 恢复常发数据帧
                    }
                } else {
                    // 仍有航点未装订
                    previousRouteBind.waitNext();
                    writeRouteBindCommand0(previousRouteBind.code, previousRouteBind.commandQueue.peek(), previousRouteBind.commandUniId);
                    log.debug("[cac] {} 当前航点装订成功，剩余航点：{}", shortInfo(), previousRouteBind.commandQueue.size());
                }
            }
            // else {
            //     if (previousRouteBind.isTimeout()) {
            //         log.info("[cac] {} 航线装订结果失败, 超过最大帧数仍未回报", shortInfo());
            //         previousRouteBind.clearCommand();
            //     }
            // }
        }
        return false;
    }
    /**开关指令帧数控制*/
    private int commandNumber(byte controlCommand) {
        switch (controlCommand) {
            case (byte) 0xA7: // 解锁
                return 45;
            case (byte) 0xE2: // 一键上锁
                return 45;
            case (byte) 0xE3: // 开伞
                return 45;
            default:
                return 8;
        }
    }


    /**Cac起飞参数查询查询回报，具体定义见各型号子类*/
    @Data
    private static class UavTakeOffParamQueryResultParam981A extends UavTakeOffParamQueryResultParam {
        /**
         * 起飞高度，单位m，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int takeoffAltitude;

        /**
         * 起飞油门，精确到整数; 原数据分辨率 1
         */
        private int takeoffThrottle;

        /**
         * 最大飞行速度，单位m/s，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int maxFlightSpeed;

        /**
         * 爬升速度限幅，单位m/s，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int climbSpeedLimit;

        /**
         * 下降速度限幅，单位m/s，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int descentSpeedLimit;

        /**
         * 返航安全高度，单位m，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int safeReturnAltitude;

        /**
         * 断链返航时间,单位min，精确到小数点后1位; 原数据分辨率 0.1
         */
        private int linkLossLaunchTime;

        /**
         * 高度上调下限，单位m，精确到整数; 原数据分辨率 1
         */
        private int minAscendingAltitude;

        /**
         * 高度下调下限，单位m，精确到整数; 原数据分辨率 1
         */
        private int minDescendingAltitude;

        /**
         * 机场航向，单位°，精确到小数点后2位; 原数据分辨率 0.1
         */
        private int runwayHeading;

        /**
         * 空投高度，单位m，精确到小数点后2位; 原数据分辨率 1
         */
        private int airDropAltitude;

        /**
         * 电池类型，0或1; 原数据分辨率 1
         */
        private int batteryType;

        public UavTakeOffParamQueryResultParam981A(ByteBuf dataBufSlice) {
            int idx = 1;
            takeoffAltitude = (int) (dataBufSlice.getUnsignedShortLE(idx) * 0.1 * 100);
            idx += 2;
            takeoffThrottle = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
            maxFlightSpeed = (int) (dataBufSlice.getUnsignedByte(idx) * 0.1 * 100);
            idx += 1;
            climbSpeedLimit = (int) (dataBufSlice.getUnsignedByte(idx) * 0.1 * 100);
            idx += 1;
            descentSpeedLimit = (int) (dataBufSlice.getByte(idx) * 0.1 * 100);
            idx += 1;
            safeReturnAltitude = (int) (dataBufSlice.getUnsignedShortLE(idx) * 0.1 * 100);
            idx += 2;
            linkLossLaunchTime = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
            minAscendingAltitude = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
            minDescendingAltitude = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
            runwayHeading = (int) (dataBufSlice.getUnsignedShortLE(idx) * 0.01 * 100);
            idx += 2;
            airDropAltitude = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
            batteryType = dataBufSlice.getUnsignedByte(idx);
            idx += 1;
        }

        public String toJsonStr() throws IOException {
            return JSONUtils.obj2json(this);
        }
    }


    @Data
    public static class UavWeightQueryResultParam981A extends UavWeightQueryResultParam{
        /**
         * 空机重量，单位kg，精确到整数位; 原数据分辨率 1
         */
        private int uavWeight;

        /**
         * 货物重量，单位kg，精确到整数位; 原数据分辨率 1
         */
        private int cargoWeight;

        public UavWeightQueryResultParam981A(ByteBuf dataBufSlice) {
            int idx = 1;
            uavWeight = dataBufSlice.getUnsignedShortLE(idx);
            idx += 2;
            cargoWeight = dataBufSlice.getUnsignedShortLE(idx);
            idx += 2;
        }
        public String toJsonStr() throws IOException {
            return JSONUtils.obj2json(this);
        }
    }
}
