package com.platform.service.handler;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.DelimiterBasedFrameDecoder;
import org.springframework.stereotype.Component;

/**
 * @Author : shiyi
 * @Date : 2025/1/15 16:00
 * @Description : 遥测数据分离， 注意分离出来的是EB 90之前的一帧，并且不含EB 90, 因此需要补上帧头
 */
@Component
public class TelemetryDecoder extends DelimiterBasedFrameDecoder {
    public static final int MAX_FRAME_LENGTH = 1024;
    public static ByteBuf DELIMITER = Unpooled.wrappedBuffer(new byte[]{(byte) 0xeb, (byte) 0x90});
    public static ByteBuf HEAD = Unpooled.wrappedBuffer(new byte[]{(byte) 0xeb, (byte) 0x90});
    public TelemetryDecoder() {
        super(MAX_FRAME_LENGTH, DELIMITER);
    }
    // public TelemetryDecoder(int maxFrameLength,  ByteBuf delimiter) {
    //     super(MAX_FRAME_LENGTH, DELIMITER);
    // }

    @Override
    protected Object decode(ChannelHandlerContext ctx, ByteBuf buffer) throws Exception {
        ByteBuf decodeBuf = (ByteBuf) super.decode(ctx, buffer);
        if (decodeBuf == null) {
            return null;
        } else if (decodeBuf.readableBytes()==0){
            decodeBuf.release(); // 释放空buffer
            return null;
        }

        try {
            // 创建组合buffer，注意HEAD需要复制因为它是静态共享的
            return Unpooled.wrappedBuffer(HEAD.retainedDuplicate(), decodeBuf);
        } catch (Exception e) {
            decodeBuf.release();
            return null; // Add return statement in catch block
        }
    }
}
