package com.platform.util;

import java.util.concurrent.atomic.AtomicInteger;

public class AtomicUByte {
    private final AtomicInteger value;

    // 构造方法初始化为 0
    public AtomicUByte() {
        value = new AtomicInteger(0);
    }

    // 获取当前值
    public byte get() {
        return (byte) value.get();  // 获取并转换为byte类型返回
    }


    // 获取当前的值并进行自增，达到255后归零
    public byte getAndIncrement() {
        while (true) {
            int current = value.get();  // 获取当前值
            int next = (current == 255) ? 0 : current + 1;  // 自增至255后归零
            // 如果当前值等于 current，则更新为 next，并返回旧值
            if (value.compareAndSet(current, next)) {
                return (byte) current;  // 返回自增前的值
            }
        }
    }

    public byte incrementAndGet() {
        while (true) {
            int current = value.get();
            int next = (current == 255) ? 0 : current + 1;
            if (value.compareAndSet(current, next)) {
                return (byte) next;
            }
        }
    }

}