package com.platform.util;

import org.apache.tomcat.util.codec.binary.Base64;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;

public class BaseAuthorizationUtils {

    public static void generateAuthAndDateHeader(Map<String, String> header, String method, String gcsId, String token, String uri) {
        Date sysdate = new Date();
        SimpleDateFormat df = new SimpleDateFormat("EEE\', \'dd\' \'MMM\' \'yyyy\' \'HH:mm:ss\' \'z", Locale.US);
        df.setTimeZone(TimeZone.getTimeZone("GMT"));
        String date = df.format(sysdate);
        header.put("authorization", generateAuth(method, date, gcsId, token, uri));
        header.put("gcsId", gcsId);
        header.put("date", date);
    }

    public static String generateAuth(String method, String date, String gcsId, String token, String uri){
        String string_to_sign = method.toUpperCase() + " " + uri + "\n" + date;
        String sig = token;
        String encoding = "";

        try {
            byte[] signature = getSignature(string_to_sign.getBytes(), sig.getBytes());
            encoding = new String(Base64.encodeBase64(signature));
        } catch (Exception var10) {
            return null;
        }

        return  gcsId + ":" + encoding;
    }

    public static byte[] getSignature(byte[] data, byte[] key) throws NoSuchAlgorithmException, InvalidKeyException {
        SecretKeySpec signingKey = new SecretKeySpec(key, "HmacSHA1");
        Mac mac = Mac.getInstance("HmacSHA1");
        mac.init(signingKey);
        byte[] rawHmac = mac.doFinal(data);
        return rawHmac;
    }

}