package com.platform.util;

import java.io.*;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.charset.StandardCharsets;
import java.util.Locale;

public class ByteUtils {

	/**
	 * 用来把mac字符串转换为long
	 *
	 * @param strMac
	 * @return
	 */
	public static long macToLong(String strMac) {
		byte[] mb = new BigInteger(strMac, 16).toByteArray();
		ByteBuffer mD = ByteBuffer.allocate(mb.length);
		mD.put(mb);
		long mac = 0;
		// 如果长度等于8代表没有补0;
		if (mD.array().length == 8) {
			mac = mD.getLong(0);
		} else if (mD.array().length == 9) {
			mac = mD.getLong(1);
		}
		return mac;
	}

	public static byte[] getBytes(Object obj) throws IOException {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(bout);
		out.writeObject(obj);
		out.flush();
		byte[] bytes = bout.toByteArray();
		bout.close();
		out.close();

		return bytes;
	}

	/**
	 * 函数名称：hexStr2Byte</br> 功能描述：String 转数组
	 *
	 * @param hex
	 */
	public static byte[] hexStr2Byte(String hex) {

		ByteBuffer bf = ByteBuffer.allocate(hex.length() / 2);
		for (int i = 0; i < hex.length(); i++) {
			String hexStr = hex.charAt(i) + "";
			i++;
			hexStr += hex.charAt(i);
			byte b = (byte) Integer.parseInt(hexStr, 16);
			bf.put(b);
		}
		return bf.array();
	}

	/**
	 * 函数名称：byteToHex</br> 功能描述：byte转16进制
	 *
	 * @param b
	 */
	public static String byteToHex(byte b) {
		String hex = Integer.toHexString(b & 0xFF);
		if (hex.length() == 1) {
			hex = '0' + hex;

		}
		return hex.toUpperCase(Locale.getDefault());

	}

	public static Object getObject(byte[] bytes) throws IOException,
			ClassNotFoundException {
		ByteArrayInputStream bi = new ByteArrayInputStream(bytes);
		ObjectInputStream oi = new ObjectInputStream(bi);
		Object obj = oi.readObject();
		bi.close();
		oi.close();
		return obj;
	}

	public static ByteBuffer getByteBuffer(Object obj) throws IOException {
		byte[] bytes = ByteUtils.getBytes(obj);
		ByteBuffer buff = ByteBuffer.wrap(bytes);

		return buff;
	}

	/**
	 * byte[] 转short 2字节
	 *
	 * @param bytes
	 * @return
	 */
	public static short bytesToshort(byte[] bytes) {
		return (short) ((bytes[0] & 0xff) | ((bytes[1] << 8) & 0xff00));

	}

	/**
	 * byte 转Int
	 *
	 * @param b
	 * @return
	 */
	public static int byteToInt(byte b) {
		return (b) & 0xff;
	}


	/**
	 * @方法功能 字节数组和整型的转换
	 * @param 字节数组
	 * @return 整型
	 */
	public static int bytesToInt(byte[] bytes,int offset) {
		int num = bytes[offset] & 0xFF;
		num |= ((bytes[offset+1] << 8) & 0xFF00);
		num |= ((bytes[offset+2] << 16) & 0xFF0000);
		num |= ((bytes[offset+3] << 24) & 0xFF000000);
		return num;
	}

	public static byte[] intToByte(int i) {

		byte[] abyte0 = new byte[4];
		abyte0[0] = (byte) (0xff & i);
		abyte0[1] = (byte) ((0xff00 & i) >> 8);
		abyte0[2] = (byte) ((0xff0000 & i) >> 16);
		abyte0[3] = (byte) ((0xff000000 & i) >> 24);
		return abyte0;

	}

	public static byte[] LongToByte(Long i) {

		byte[] abyte0 = new byte[8];
		abyte0[0] = (byte) (0xff & i);
		abyte0[1] = (byte) ((0xff00 & i) >> 8);
		abyte0[2] = (byte) ((0xff0000 & i) >> 16);
		abyte0[3] = (byte) ((0xff000000 & i) >> 24);
		abyte0[4] = (byte) ((0xff00000000l & i) >> 32);
		abyte0[5] = (byte) ((0xff0000000000l & i) >> 40);
		abyte0[6] = (byte) ((0xff000000000000l & i) >> 48);
		abyte0[7] = (byte) ((0xff00000000000000l & i) >> 56);
		return abyte0;

	}

	/**
	 * 函数名称：shortChange</br> 功能描述：short 大端转小端
	 *
	 * @param mshort
	 */
	public static short shortChange(Short mshort) {

		mshort = (short) ((mshort >> 8 & 0xFF) | (mshort << 8 & 0xFF00));

		return mshort;
	}

	/**
	 * 函数名称：intChange</br> 功能描述：int 大端转小端
	 *
	 * @param mint
	 */
	public static int intChange(int mint) {

		mint = (int) (((mint) >> 24 & 0xFF) | ((mint) >> 8 & 0xFF00)
				| ((mint) << 8 & 0xFF0000) | ((mint) << 24 & 0xFF000000));

		return mint;
	}

	/**
	 * 函数名称：intChange</br> 功能描述：LONG 大端转小端
	 *
	 * @param mint
	 */
	public static long longChange(long mint) {

		mint = (long) (((mint) >> 56 & 0xFF) | ((mint) >> 48 & 0xFF00)
				| ((mint) >> 24 & 0xFF0000) | ((mint) >> 8 & 0xFF000000)
				| ((mint) << 8 & 0xFF00000000l)
				| ((mint) << 24 & 0xFF0000000000l)
				| ((mint) << 40 & 0xFF000000000000l) | ((mint) << 56 & 0xFF00000000000000l));

		return mint;
	}

	/**
	 * 将byte转换为无符号的short类型
	 *
	 * @param b
	 *            需要转换的字节数
	 * @return 转换完成的short
	 */
	public static short byteToUshort(byte b) {
		return (short) (b & 0x00ff);
	}

	/**
	 * 将byte转换为无符号的int类型
	 *
	 * @param b
	 *            需要转换的字节数
	 * @return 转换完成的int
	 */
	public static int byteToUint(byte b) {
		return b & 0x00ff;
	}

	/**
	 * 将byte转换为无符号的long类型
	 *
	 * @param b
	 *            需要转换的字节数
	 * @return 转换完成的long
	 */
	public static long byteToUlong(byte b) {
		return b & 0x00ff;
	}

	/**
	 * 将short转换为无符号的int类型
	 *
	 * @param s
	 *            需要转换的short
	 * @return 转换完成的int
	 */
	public static int shortToUint(short s) {
		return s & 0x00ffff;
	}

	/**
	 * 将short转换为无符号的long类型
	 *
	 * @param s
	 *            需要转换的字节数
	 * @return 转换完成的long
	 */
	public static long shortToUlong(short s) {
		return s & 0x00ffff;
	}

	/**
	 * 将int转换为无符号的long类型
	 *
	 * @param i
	 *            需要转换的字节数
	 * @return 转换完成的long
	 */
	public static long intToUlong(int i) {
		return i & 0x00ffffffff;
	}

	/**
	 * 将short转换成小端序的byte数组
	 *
	 * @param s
	 *            需要转换的short
	 * @return 转换完成的byte数组
	 */
	public static byte[] shortToLittleEndianByteArray(short s) {
		return ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN)
				.putShort(s).array();
	}

	/**
	 * 将int转换成小端序的byte数组
	 *
	 * @param i
	 *            需要转换的int
	 * @return 转换完成的byte数组
	 */
	public static byte[] intToLittleEndianByteArray(int i) {
		return ByteBuffer.allocate(4).order(ByteOrder.LITTLE_ENDIAN).putInt(i)
				.array();
	}

	/**
	 * 将long转换成小端序的byte数组
	 *
	 * @param l
	 *            需要转换的long
	 * @return 转换完成的byte数组
	 */
	public static byte[] longToLittleEndianByteArray(long l) {
		return ByteBuffer.allocate(8).order(ByteOrder.LITTLE_ENDIAN).putLong(l)
				.array();
	}

	/**
	 * 将short转换成大端序的byte数组
	 *
	 * @param s
	 *            需要转换的short
	 * @return 转换完成的byte数组
	 */
	public static byte[] shortToBigEndianByteArray(short s) {
		return ByteBuffer.allocate(2).order(ByteOrder.BIG_ENDIAN).putShort(s)
				.array();
	}

	/**
	 * 将int转换成大端序的byte数组
	 *
	 * @param i
	 *            需要转换的int
	 * @return 转换完成的byte数组
	 */
	public static byte[] intToBigEndianByteArray(int i) {
		return ByteBuffer.allocate(2).order(ByteOrder.BIG_ENDIAN).putInt(i)
				.array();
	}

	/**
	 * 将long转换成大端序的byte数组
	 *
	 * @param l
	 *            需要转换的long
	 * @return 转换完成的byte数组
	 */
	public static byte[] longToBigEndianByteArray(long l) {
		return ByteBuffer.allocate(2).order(ByteOrder.BIG_ENDIAN).putLong(l)
				.array();
	}

	/**
	 * 将short转换为16进制字符串
	 *
	 * @param s
	 *            需要转换的short
	 * @param isLittleEndian
	 *            是否是小端序（true为小端序false为大端序）
	 * @return 转换后的字符串
	 */
	public static String shortToHexString(short s, boolean isLittleEndian) {
		byte byteArray[] = null;
		if (isLittleEndian) {
			byteArray = shortToLittleEndianByteArray(s);
		} else {
			byteArray = shortToBigEndianByteArray(s);
		}
		return byteArrayToHexString(byteArray);
	}

	/**
	 * 将int转换为16进制字符串
	 *
	 * @param i
	 *            需要转换的int
	 * @param isLittleEndian
	 *            是否是小端序（true为小端序false为大端序）
	 * @return 转换后的字符串
	 */
	public static String intToHexString(int i, boolean isLittleEndian) {
		byte byteArray[] = null;
		if (isLittleEndian) {
			byteArray = intToLittleEndianByteArray(i);
		} else {
			byteArray = intToBigEndianByteArray(i);
		}
		return byteArrayToHexString(byteArray);
	}

	/**
	 * 将long转换为16进制字符串
	 *
	 * @param l
	 *            需要转换的long
	 * @param isLittleEndian
	 *            是否是小端序（true为小端序false为大端序）
	 * @return 转换后的字符串
	 */
	public static String longToHexString(long l, boolean isLittleEndian) {
		byte byteArray[] = null;
		if (isLittleEndian) {
			byteArray = longToLittleEndianByteArray(l);
		} else {
			byteArray = longToBigEndianByteArray(l);
		}
		return byteArrayToHexString(byteArray);
	}

	/**
	 * 将字节数组转换成16进制字符串
	 *
	 * @param array
	 *            需要转换的字符串
	 * @param toPrint
	 *            是否为了打印输出，如果为true则会每4自己添加一个空格
	 * @return 转换完成的字符串
	 */
	public static String byteArrayToHexString(byte[] array, boolean toPrint) {
		if (array == null) {
			return "null";
		}
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < array.length; i++) {
			sb.append(byteToHex(array[i]));
			if (toPrint && (i + 1) % 4 == 0) {
				sb.append(" ");
			}
		}
		return sb.toString();
	}

	/**
	 * 将字节数组转换成16进制字符串
	 *
	 * @param array
	 *            需要转换的字符串(字节间没有分隔符)
	 * @return 转换完成的字符串
	 */
	public static String byteArrayToHexString(byte[] array) {
		return byteArrayToHexString(array, false);
	}


	public static String bytes2HexString(byte[] b) {
		String ret = "";
		for (int i = 0; i < b.length; i++) {
			String hex = Integer.toHexString(b[ i ] & 0xFF);
			if (hex.length() == 1) {
				hex = '0' + hex;
			}
			ret += hex.toUpperCase()+" ";
		}
		return ret;
	}

	/**
	 * 将字节数组转换成long类型
	 *
	 * @param bytes
	 *            字节数据
	 * @return long类型
	 */
	public static long byteArrayToLong(byte[] bytes) {
		return ((((long) bytes[0] & 0xff) << 24)
				| (((long) bytes[1] & 0xff) << 16)
				| (((long) bytes[2] & 0xff) << 8) | (((long) bytes[3] & 0xff) << 0));
	}



	public static byte[] getCRC(int[] ptr){
		int[] crc_16_table = {
				0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
				0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
				0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
				0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
				0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
				0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
				0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
				0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
				0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823,
				0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
				0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12,
				0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
				0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41,
				0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
				0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70,
				0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
				0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
				0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
				0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
				0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
				0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
				0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
				0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
				0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
				0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
				0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3,
				0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
				0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92,
				0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
				0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1,
				0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
				0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
		};
		int crc = 0x0000;
		int da = 0x00;
		for (int j : ptr) {
			da = (crc & 0xff00) >> 8;
			crc = (crc & 0x00ff) << 8;
			int a = da ^ j;
			int b = crc_16_table[a];
			crc ^=b;
		}


		return intToLittleEndianByteArray(crc);
	}




	/**
	 * 计算CRC16校验码
	 *
	 * @param bytes
	 * @return
	 */
	public static byte[] getRtuCRC(int[] bytes) {
		int CRC = 0x0000ffff;
		int POLYNOMIAL = 0x0000a001;

		int i, j;
		for (i = 0; i < bytes.length; i++) {
			CRC ^= (bytes[i] & 0x000000ff);
			for (j = 0; j < 8; j++) {
				if ((CRC & 0x00000001) != 0) {
					CRC >>= 1;
					CRC ^= POLYNOMIAL;
				} else {
					CRC >>= 1;
				}
			}
		}
//		CRC = ( (CRC & 0x0000FF00) >> 8) | ( (CRC & 0x000000FF ) << 8);
		return intToLittleEndianByteArray(CRC);
	}

	/**
	 * 从ByteBuffer读取指定长度的字符串
	 * @param buffer
	 * @param length
	 * @return
	 */
	public static String getString(ByteBuffer buffer, int length){
		byte[] bytes = new byte[length];
		buffer.get(bytes);
		return new String(bytes, StandardCharsets.UTF_8);
	}
}
